@@ -1,7 +1,10 @@
-use super::{Context, Input, Lexer};
+use super::{
+    comments_buffer::{BufferedComment, BufferedCommentKind},
+    Context, Input, Lexer,
+};
 use crate::{error::Error, input::Tokens, lexer::util::CharExt, token::*, JscTarget, Syntax};
 use enum_kind::Kind;
-use std::{mem, mem::take};
+use std::mem::take;
 use swc_common::BytePos;
 use tracing::trace;
 
@@ -182,30 +185,39 @@ impl<'a, I: Input> Iterator for Lexer<'a, I> {
                 Some(c) => c,
                 // End of input.
                 None => {
-                    if self
-                        .leading_comments_buffer
-                        .as_ref()
-                        .map(|v| !v.borrow().is_empty())
-                        .unwrap_or(false)
-                    {
+                    if let Some(comments) = self.comments.as_mut() {
+                        let comments_buffer = self.comments_buffer.as_mut().unwrap();
                         let last = self.state.prev_hi;
 
-                        for c in self
-                            .leading_comments_buffer
-                            .as_ref()
-                            .map(|v| v.borrow_mut())
-                            .unwrap()
-                            .drain(..)
-                        {
-                            let comments = self.comments.as_mut().unwrap();
-
+                        // move the pending to the leading or trailing
+                        for c in comments_buffer.take_pending_leading() {
                             // if the file had no tokens and no shebang, then treat any
                             // comments in the leading comments buffer as leading.
                             // Otherwise treat them as trailing.
                             if last == BytePos(0) {
-                                comments.add_leading(last, c);
+                                comments_buffer.push(BufferedComment {
+                                    kind: BufferedCommentKind::Leading,
+                                    pos: last,
+                                    comment: c,
+                                });
                             } else {
-                                comments.add_trailing(last, c);
+                                comments_buffer.push(BufferedComment {
+                                    kind: BufferedCommentKind::Trailing,
+                                    pos: last,
+                                    comment: c,
+                                });
+                            }
+                        }
+
+                        // now fill the user's passed in comments
+                        for comment in comments_buffer.take_comments() {
+                            match comment.kind {
+                                BufferedCommentKind::Leading => {
+                                    comments.add_leading(comment.pos, comment.comment);
+                                }
+                                BufferedCommentKind::Trailing => {
+                                    comments.add_trailing(comment.pos, comment.comment);
+                                }
                             }
                         }
                     }
@@ -283,25 +295,14 @@ impl<'a, I: Input> Iterator for Lexer<'a, I> {
 
         let span = self.span(start);
         if let Some(ref token) = token {
-            if self.leading_comments_buffer.is_some()
-                && !self
-                    .leading_comments_buffer
-                    .as_ref()
-                    .unwrap()
-                    .borrow()
-                    .is_empty()
-            {
-                self.comments.as_ref().unwrap().add_leading_comments(
-                    start,
-                    mem::replace(
-                        &mut *self
-                            .leading_comments_buffer
-                            .as_ref()
-                            .map(|v| v.borrow_mut())
-                            .unwrap(),
-                        vec![],
-                    ),
-                );
+            if let Some(comments) = self.comments_buffer.as_mut() {
+                for comment in comments.take_pending_leading() {
+                    comments.push(BufferedComment {
+                        kind: BufferedCommentKind::Leading,
+                        pos: start,
+                        comment,
+                    });
+                }
             }
             self.state.update(start, &token);
             self.state.prev_hi = self.last_pos();
